#ifndef Mesh_h
#define Mesh_h

#include <list>
using std::list;
#include <valarray>
using std::valarray;

#include "TVecMat.h"
#include "Vertex.h"
#include "BBox3d.h"

#include "DepthMap.h"

struct MeshMode
{
	int _ascii, _hasConf, _hasColor, _hasMask, _hasTris;
	//these numbers determine the ply IO procedure:
	//1: yes
	//0: no
	//-1: unknown, depending on the input file, used in readPly only

	MeshMode()
	{ _ascii = _hasConf = _hasColor = _hasMask = _hasTris = -1; }
};

template <class floating>
class Mesh 
{

public:

	string _name;

    valarray< Vertex<floating> > _verts;
	valarray< Triangle<floating> > _tris;
	BBox3d _bbox;
	bool _hasNormals;

	string Name() const
	{ return _name; }
	bool empty() const
	{ return NumVerts() == 0; }
	int NumVerts() const
	{ return (int)_verts.size(); }
	int NumTris() const
	{ return (int)_tris.size(); }
	BBox3d BBox() const
	{ return _bbox; }

	bool ValidTriangleIndex(int i)
	{ return 0 <= i && i < NumTris(); }
	bool ValidVertexIndex(int i)
	{ return 0 <= i && i < NumVerts(); }

	void Free();

    // Compute axis-aligned bounding box for whole mesh
    void computeBBox();
	// Compute triangle and vertex normals.
    void initNormals();
    // Compute triangle normals.
    void computeTriNormals();
    // Compute vertex normals as average of triangle normals.
    void computeVertNormals();
	void FlipTriNormals();
	void FlipVertNormals();
	
	bool CreateFromDepthMap(const DepthMap &depth, double gap = 10);
    
	bool readPly(const char *filename, MeshMode *meshMode);
    bool writePly(const char *filename, const MeshMode *meshMode) const;	

	static void computeTriNormals(const valarray< Vertex<floating> > &verts, valarray< Triangle<floating> > &tris);
	static void computeVertNormals(valarray< Vertex<floating> > &verts, const valarray< Triangle<floating> > &tris);
	static void initNormals(valarray< Vertex<floating> > &verts, valarray< Triangle<floating> > &tris);
};

#endif
